@@ -1,10 +1,11 @@
-//go:build cgo && !nodagre
+//go:build !nodagre
 
 package main
 
 import (
+	"oss.terrastruct.com/util-go/xmain"
+
 	"oss.terrastruct.com/d2/d2plugin"
-	"oss.terrastruct.com/d2/lib/xmain"
 )
 
 func main() {
