@@ -8,6 +8,8 @@ import (
 	"strconv"
 	"strings"
 
+	"oss.terrastruct.com/util-go/go2"
+
 	"oss.terrastruct.com/d2/d2ast"
 	"oss.terrastruct.com/d2/d2format"
 	"oss.terrastruct.com/d2/d2graph"
@@ -250,11 +252,8 @@ func (c *compiler) compileKey(obj *d2graph.Object, m *d2ast.Map, mk *d2ast.Key)
 		c.errorf(mk.Range.Start, mk.Range.End, err.Error())
 		return
 	}
-	if resolvedObj != obj {
-		obj = resolvedObj
-	}
 
-	parent := obj
+	parent := resolvedObj
 	if len(resolvedIDA) > 0 {
 		unresolvedObj := obj
 		obj = parent.EnsureChild(resolvedIDA)
@@ -266,6 +265,8 @@ func (c *compiler) compileKey(obj *d2graph.Object, m *d2ast.Map, mk *d2ast.Key)
 		}, unresolvedObj)
 	} else if obj.Parent == nil {
 		// Top level reserved key set on root.
+		c.compileAttributes(&obj.Attributes, mk)
+		c.applyScalar(&obj.Attributes, reserved, mk.Value.ScalarBox())
 		return
 	}
 
@@ -354,6 +355,14 @@ func (c *compiler) applyScalar(attrs *d2graph.Attributes, reserved string, box d
 	case "link":
 		attrs.Link = scalar.ScalarString()
 		return
+	case "direction":
+		dirs := []string{"up", "down", "right", "left"}
+		if !go2.Contains(dirs, scalar.ScalarString()) {
+			c.errorf(scalar.GetRange().Start, scalar.GetRange().End, `direction must be one of %v, got %q`, strings.Join(dirs, ", "), scalar.ScalarString())
+			return
+		}
+		attrs.Direction.Value = scalar.ScalarString()
+		return
 	}
 
 	if _, ok := d2graph.StyleKeywords[reserved]; ok {
@@ -377,7 +386,7 @@ func (c *compiler) applyScalar(attrs *d2graph.Attributes, reserved string, box d
 		if ok {
 			attrs.Language = fullTag
 		}
-		if attrs.Language == "markdown" {
+		if attrs.Language == "markdown" || attrs.Language == "latex" {
 			attrs.Shape.Value = d2target.ShapeText
 		} else {
 			attrs.Shape.Value = d2target.ShapeCode
@@ -548,12 +557,13 @@ func (c *compiler) compileFlatKey(k *d2ast.KeyPath) ([]string, string, bool) {
 
 // TODO add more, e.g. C, bash
 var ShortToFullLanguageAliases = map[string]string{
-	"md": "markdown",
-	"js": "javascript",
-	"go": "golang",
-	"py": "python",
-	"rb": "ruby",
-	"ts": "typescript",
+	"md":  "markdown",
+	"tex": "latex",
+	"js":  "javascript",
+	"go":  "golang",
+	"py":  "python",
+	"rb":  "ruby",
+	"ts":  "typescript",
 }
 var FullToShortLanguageAliases map[string]string
 
