@@ -6,9 +6,9 @@ import (
 	"strings"
 	"testing"
 
-	"oss.terrastruct.com/diff"
-
-	"github.com/stretchr/testify/assert"
+	tassert "github.com/stretchr/testify/assert"
+	"oss.terrastruct.com/util-go/assert"
+	"oss.terrastruct.com/util-go/diff"
 
 	"oss.terrastruct.com/d2/d2compiler"
 	"oss.terrastruct.com/d2/d2format"
@@ -210,6 +210,20 @@ x: {
 
 			},
 		},
+		{
+			name: "underscore_unresolved_obj",
+
+			text: `
+x: {
+	_.y
+}
+`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				tassert.Equal(t, "y", g.Objects[1].ID)
+				tassert.Equal(t, g.Root.AbsID(), g.Objects[1].References[0].ScopeObj.AbsID())
+				tassert.Equal(t, g.Objects[0].AbsID(), g.Objects[1].References[0].UnresolvedScopeObj.AbsID())
+			},
+		},
 		{
 			name: "underscore_parent_not_root",
 
@@ -775,8 +789,8 @@ x -> y: {
 				if len(g.Objects) != 2 {
 					t.Fatalf("expected 2 objects: %#v", g.Objects)
 				}
-				diff.AssertStringEq(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
-				assert.Empty(t, g.Edges[0].Attributes.Shape.Value)
+				assert.String(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
+				assert.String(t, "", g.Edges[0].Attributes.Shape.Value)
 				// Make sure the DSL didn't change. this is a regression test where it did
 				exp := `x -> y: {
   source-arrowhead: {
@@ -814,13 +828,13 @@ x -> y: {
 				if len(g.Objects) != 2 {
 					t.Fatalf("expected 2 objects: %#v", g.Objects)
 				}
-				diff.AssertStringEq(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
-				diff.AssertStringEq(t, "Reisner's Rule of Conceptual Inertia", g.Edges[0].SrcArrowhead.Label.Value)
-				diff.AssertStringEq(t, "QOTD", g.Edges[0].DstArrowhead.Label.Value)
-				diff.AssertStringEq(t, "true", g.Edges[0].DstArrowhead.Style.Filled.Value)
-				assert.Empty(t, g.Edges[0].Attributes.Shape.Value)
-				assert.Empty(t, g.Edges[0].Attributes.Label.Value)
-				assert.Nil(t, g.Edges[0].Attributes.Style.Filled)
+				assert.String(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
+				assert.String(t, "Reisner's Rule of Conceptual Inertia", g.Edges[0].SrcArrowhead.Label.Value)
+				assert.String(t, "QOTD", g.Edges[0].DstArrowhead.Label.Value)
+				assert.String(t, "true", g.Edges[0].DstArrowhead.Style.Filled.Value)
+				assert.String(t, "", g.Edges[0].Attributes.Shape.Value)
+				assert.String(t, "", g.Edges[0].Attributes.Label.Value)
+				assert.JSON(t, nil, g.Edges[0].Attributes.Style.Filled)
 			},
 		},
 		{
@@ -836,8 +850,8 @@ x -> y: {
 				if len(g.Objects) != 2 {
 					t.Fatalf("expected 2 objects: %#v", g.Objects)
 				}
-				diff.AssertStringEq(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
-				assert.Empty(t, g.Edges[0].Attributes.Shape.Value)
+				assert.String(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
+				assert.String(t, "", g.Edges[0].Attributes.Shape.Value)
 			},
 		},
 		{
@@ -853,8 +867,8 @@ x -> y: {
 				if len(g.Objects) != 2 {
 					t.Fatalf("expected 2 objects: %#v", g.Objects)
 				}
-				diff.AssertStringEq(t, "triangle", g.Edges[0].SrcArrowhead.Shape.Value)
-				assert.Empty(t, g.Edges[0].Attributes.Shape.Value)
+				assert.String(t, "triangle", g.Edges[0].SrcArrowhead.Shape.Value)
+				assert.String(t, "", g.Edges[0].Attributes.Shape.Value)
 			},
 		},
 		{
@@ -880,8 +894,8 @@ x -> y: {
 				if len(g.Objects) != 2 {
 					t.Fatalf("expected 2 objects: %#v", g.Objects)
 				}
-				diff.AssertStringEq(t, "yo", g.Edges[0].SrcArrowhead.Label.Value)
-				assert.Empty(t, g.Edges[0].Attributes.Label.Value)
+				assert.String(t, "yo", g.Edges[0].SrcArrowhead.Label.Value)
+				assert.String(t, "", g.Edges[0].Attributes.Label.Value)
 			},
 		},
 		{
@@ -899,8 +913,8 @@ x -> y: {
 				if len(g.Objects) != 2 {
 					t.Fatalf("expected 2 objects: %#v", g.Objects)
 				}
-				diff.AssertStringEq(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
-				assert.Empty(t, g.Edges[0].Attributes.Shape.Value)
+				assert.String(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
+				assert.String(t, "", g.Edges[0].Attributes.Shape.Value)
 			},
 		},
 		{
@@ -920,9 +934,9 @@ x -> y: {
 				if len(g.Objects) != 2 {
 					t.Fatalf("expected 2 objects: %#v", g.Objects)
 				}
-				diff.AssertStringEq(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
-				diff.AssertStringEq(t, "diamond", g.Edges[0].DstArrowhead.Shape.Value)
-				assert.Empty(t, g.Edges[0].Attributes.Shape.Value)
+				assert.String(t, "diamond", g.Edges[0].SrcArrowhead.Shape.Value)
+				assert.String(t, "diamond", g.Edges[0].DstArrowhead.Shape.Value)
+				assert.String(t, "", g.Edges[0].Attributes.Shape.Value)
 			},
 		},
 		{
@@ -1333,10 +1347,23 @@ y -> x.style
 				if len(g.Objects) != 1 {
 					t.Fatal(g.Objects)
 				}
-				diff.AssertStringEq(t, `b
+				assert.String(t, `"b\nb"`, g.Objects[0].ID)
+				assert.String(t, `b
 b`, g.Objects[0].Attributes.Label.Value)
 			},
 		},
+		{
+			name: "unescaped_id_cr",
+
+			text: `b\rb`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				if len(g.Objects) != 1 {
+					t.Fatal(g.Objects)
+				}
+				assert.String(t, "b\rb", g.Objects[0].ID)
+				assert.String(t, "b\rb", g.Objects[0].Attributes.Label.Value)
+			},
+		},
 		{
 			name: "class_style",
 
@@ -1420,9 +1447,9 @@ b`, g.Objects[0].Attributes.Label.Value)
 				if len(g.Objects) != 1 {
 					t.Fatal(g.Objects)
 				}
-				diff.AssertStringEq(t, `field here`, g.Objects[0].Class.Fields[0].Name)
-				diff.AssertStringEq(t, `GetType()`, g.Objects[0].Class.Methods[0].Name)
-				diff.AssertStringEq(t, `Is()`, g.Objects[0].Class.Methods[1].Name)
+				assert.String(t, `field here`, g.Objects[0].Class.Fields[0].Name)
+				assert.String(t, `GetType()`, g.Objects[0].Class.Methods[0].Name)
+				assert.String(t, `Is()`, g.Objects[0].Class.Methods[1].Name)
 			},
 		},
 		{
@@ -1438,8 +1465,8 @@ b`, g.Objects[0].Attributes.Label.Value)
 				if len(g.Objects) != 1 {
 					t.Fatal(g.Objects)
 				}
-				diff.AssertStringEq(t, `GetType()`, g.Objects[0].SQLTable.Columns[0].Name)
-				diff.AssertStringEq(t, `Is()`, g.Objects[0].SQLTable.Columns[1].Name)
+				assert.String(t, `GetType()`, g.Objects[0].SQLTable.Columns[0].Name)
+				assert.String(t, `Is()`, g.Objects[0].SQLTable.Columns[1].Name)
 			},
 		},
 		{
@@ -1463,8 +1490,8 @@ b`, g.Objects[0].Attributes.Label.Value)
 				if len(g.Objects[0].ChildrenArray) != 1 {
 					t.Fatal(g.Objects)
 				}
-				diff.AssertStringEq(t, `GetType()`, g.Objects[1].SQLTable.Columns[0].Name)
-				diff.AssertStringEq(t, `Is()`, g.Objects[1].SQLTable.Columns[1].Name)
+				assert.String(t, `GetType()`, g.Objects[1].SQLTable.Columns[0].Name)
+				assert.String(t, `Is()`, g.Objects[1].SQLTable.Columns[1].Name)
 			},
 		},
 		{
@@ -1501,6 +1528,117 @@ dst.id <-> src.dst_id
 				}
 			},
 		},
+		{
+			name: "basic_sequence",
+
+			text: `x: {
+  shape: sequence_diagram
+}
+`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				assert.String(t, "sequence_diagram", g.Objects[0].Attributes.Shape.Value)
+			},
+		},
+		{
+			name: "root_sequence",
+
+			text: `shape: sequence_diagram
+`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				assert.String(t, "sequence_diagram", g.Root.Attributes.Shape.Value)
+			},
+		},
+		{
+			name: "leaky_sequence",
+
+			text: `x: {
+  shape: sequence_diagram
+  a
+}
+b -> x.a
+`,
+			expErr: `d2/testdata/d2compiler/TestCompile/leaky_sequence.d2:5:1: connections within sequence diagrams can connect only to other objects within the same sequence diagram
+`,
+		},
+		{
+			name: "sequence_scoping",
+
+			text: `x: {
+  shape: sequence_diagram
+	a;b
+  group: {
+    a -> b
+    a.t1 -> b.t1
+    b.t1.t2 -> b.t1
+  }
+}
+`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				tassert.Equal(t, 7, len(g.Objects))
+				tassert.Equal(t, 3, len(g.Objects[0].ChildrenArray))
+			},
+		},
+		{
+			name: "sequence_grouped_note",
+
+			text: `shape: sequence_diagram
+a;d
+choo: {
+  d."this note"
+}
+`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				tassert.Equal(t, 4, len(g.Objects))
+				tassert.Equal(t, 3, len(g.Root.ChildrenArray))
+			},
+		},
+		{
+			name: "root_direction",
+
+			text: `direction: right`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				assert.String(t, "right", g.Root.Attributes.Direction.Value)
+			},
+		},
+		{
+			name: "default_direction",
+
+			text: `x`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				assert.String(t, "", g.Objects[0].Attributes.Direction.Value)
+			},
+		},
+		{
+			name: "set_direction",
+
+			text: `x: {
+  direction: left
+}`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				assert.String(t, "left", g.Objects[0].Attributes.Direction.Value)
+			},
+		},
+		{
+			name: "invalid_direction",
+
+			text: `x: {
+  direction: diagonal
+}`,
+			expErr: `d2/testdata/d2compiler/TestCompile/invalid_direction.d2:2:14: direction must be one of up, down, right, left, got "diagonal"
+`,
+		},
+		{
+			name: "self-referencing",
+
+			text: `x -> x
+`,
+			assertions: func(t *testing.T, g *d2graph.Graph) {
+				_, err := diff.Strings(g.Edges[0].Dst.ID, g.Edges[0].Src.ID)
+				if err != nil {
+					t.Fatal(err)
+				}
+			},
+		},
 	}
 
 	for _, tc := range testCases {
@@ -1539,10 +1677,8 @@ dst.id <-> src.dst_id
 				Err:   err,
 			}
 
-			err = diff.Testdata(filepath.Join("..", "testdata", "d2compiler", t.Name()), got)
-			if err != nil {
-				t.Fatal(err)
-			}
+			err = diff.TestdataJSON(filepath.Join("..", "testdata", "d2compiler", t.Name()), got)
+			assert.Success(t, err)
 		})
 	}
 }
