@@ -7,17 +7,21 @@ import (
 	"strconv"
 	"strings"
 
+	"oss.terrastruct.com/util-go/go2"
+
 	"oss.terrastruct.com/d2/d2ast"
 	"oss.terrastruct.com/d2/d2format"
 	"oss.terrastruct.com/d2/d2parser"
 	"oss.terrastruct.com/d2/d2renderers/d2fonts"
-	"oss.terrastruct.com/d2/d2renderers/textmeasure"
+	"oss.terrastruct.com/d2/d2renderers/d2latex"
 	"oss.terrastruct.com/d2/d2target"
 	"oss.terrastruct.com/d2/d2themes"
 	"oss.terrastruct.com/d2/lib/geo"
-	"oss.terrastruct.com/d2/lib/go2"
+	"oss.terrastruct.com/d2/lib/textmeasure"
 )
 
+const INNER_LABEL_PADDING int = 5
+
 // TODO: Refactor with a light abstract layer on top of AST implementing scenarios,
 // variables, imports, substitutions and then a final set of structures representing
 // a final graph.
@@ -77,6 +81,8 @@ type Object struct {
 	ChildrenArray []*Object          `json:"-"`
 
 	Attributes Attributes `json:"attributes"`
+
+	ZIndex int `json:"zIndex"`
 }
 
 type Attributes struct {
@@ -96,6 +102,8 @@ type Attributes struct {
 	Language string         `json:"language,omitempty"`
 	// TODO: default to ShapeRectangle instead of empty string
 	Shape Scalar `json:"shape"`
+
+	Direction Scalar `json:"direction"`
 }
 
 // TODO references at the root scope should have their Scope set to root graph AST
@@ -299,18 +307,6 @@ func (s *Style) Apply(key, value string) error {
 
 type ContainerLevel int
 
-func (l ContainerLevel) Fill() string {
-	// Darkest (least nested) to lightest (most nested)
-	if l == 1 {
-		return "#E3E9FD"
-	} else if l == 2 {
-		return "#EDF0FD"
-	} else if l == 3 {
-		return "#F7F8FE"
-	}
-	return "#FFFFFF"
-}
-
 func (l ContainerLevel) LabelSize() int {
 	// Largest to smallest
 	if l == 1 {
@@ -325,6 +321,34 @@ func (l ContainerLevel) LabelSize() int {
 
 func (obj *Object) GetFill(theme *d2themes.Theme) string {
 	level := int(obj.Level())
+	if obj.IsSequenceDiagramNote() {
+		return theme.Colors.Neutrals.N7
+	} else if obj.IsSequenceDiagramGroup() {
+		return theme.Colors.Neutrals.N5
+	} else if obj.Parent.IsSequenceDiagram() {
+		return theme.Colors.B5
+	}
+
+	// fill for spans
+	sd := obj.OuterSequenceDiagram()
+	if sd != nil {
+		level -= int(sd.Level())
+		if level == 1 {
+			return theme.Colors.B3
+		} else if level == 2 {
+			return theme.Colors.B4
+		} else if level == 3 {
+			return theme.Colors.B5
+		} else if level == 4 {
+			return theme.Colors.Neutrals.N6
+		}
+		return theme.Colors.Neutrals.N7
+	}
+
+	if obj.IsSequenceDiagram() {
+		return theme.Colors.Neutrals.N7
+	}
+
 	shape := obj.Attributes.Shape.Value
 
 	if shape == "" || strings.EqualFold(shape, d2target.ShapeSquare) || strings.EqualFold(shape, d2target.ShapeCircle) || strings.EqualFold(shape, d2target.ShapeOval) || strings.EqualFold(shape, d2target.ShapeRectangle) {
@@ -408,9 +432,17 @@ func (obj *Object) AbsIDArray() []string {
 }
 
 func (obj *Object) Text() *d2target.MText {
+	isBold := !obj.IsContainer()
 	fontSize := d2fonts.FONT_SIZE_M
-	if obj.IsContainer() {
-		fontSize = obj.Level().LabelSize()
+	if obj.OuterSequenceDiagram() == nil {
+		if obj.IsContainer() {
+			fontSize = obj.Level().LabelSize()
+		}
+	} else {
+		isBold = false
+	}
+	if obj.Attributes.Style.FontSize != nil {
+		fontSize, _ = strconv.Atoi(obj.Attributes.Style.FontSize.Value)
 	}
 	// Class and Table objects have Label set to header
 	if obj.Class != nil || obj.SQLTable != nil {
@@ -419,7 +451,7 @@ func (obj *Object) Text() *d2target.MText {
 	return &d2target.MText{
 		Text:     obj.Attributes.Label.Value,
 		FontSize: fontSize,
-		IsBold:   !obj.IsContainer(),
+		IsBold:   isBold,
 		IsItalic: false,
 		Language: obj.Attributes.Language,
 		Shape:    obj.Attributes.Shape.Value,
@@ -495,6 +527,22 @@ func (obj *Object) HasEdge(mk *d2ast.Key) (*Edge, bool) {
 }
 
 func ResolveUnderscoreKey(ida []string, obj *Object) (resolvedObj *Object, resolvedIDA []string, _ error) {
+	if len(ida) > 0 && !obj.IsSequenceDiagram() {
+		objSD := obj.OuterSequenceDiagram()
+		if objSD != nil {
+			referencesActor := false
+			for _, c := range objSD.ChildrenArray {
+				if c.ID == ida[0] {
+					referencesActor = true
+					break
+				}
+			}
+			if referencesActor {
+				obj = objSD
+			}
+		}
+	}
+
 	resolvedObj = obj
 	resolvedIDA = ida
 
@@ -627,6 +675,8 @@ type Edge struct {
 
 	References []EdgeReference `json:"references,omitempty"`
 	Attributes Attributes      `json:"attributes"`
+
+	ZIndex int `json:"zIndex"`
 }
 
 type EdgeReference struct {
@@ -659,9 +709,13 @@ func (e *Edge) ArrowString() string {
 }
 
 func (e *Edge) Text() *d2target.MText {
+	fontSize := d2fonts.FONT_SIZE_M
+	if e.Attributes.Style.FontSize != nil {
+		fontSize, _ = strconv.Atoi(e.Attributes.Style.FontSize.Value)
+	}
 	return &d2target.MText{
 		Text:     e.Attributes.Label.Value,
-		FontSize: d2fonts.FONT_SIZE_M,
+		FontSize: fontSize,
 		IsBold:   false,
 		IsItalic: true,
 
@@ -712,8 +766,8 @@ func (obj *Object) Connect(srcID, dstID []string, srcArrow, dstArrow bool, label
 	src := srcObj.EnsureChild(srcID)
 	dst := dstObj.EnsureChild(dstID)
 
-	if src == dst {
-		return nil, errors.New("self-referencing connection")
+	if src.OuterSequenceDiagram() != dst.OuterSequenceDiagram() {
+		return nil, errors.New("connections within sequence diagrams can connect only to other objects within the same sequence diagram")
 	}
 
 	edge := &Edge{
@@ -831,12 +885,20 @@ func (g *Graph) SetDimensions(mtexts []*d2target.MText, ruler *textmeasure.Ruler
 		}
 
 		var dims *d2target.TextDimensions
-		var innerLabelPadding = 5
+		var innerLabelPadding = INNER_LABEL_PADDING
 		if obj.Attributes.Shape.Value == d2target.ShapeText {
-			var err error
-			dims, err = getMarkdownDimensions(mtexts, ruler, obj.Text())
-			if err != nil {
-				return err
+			if obj.Attributes.Language == "latex" {
+				width, height, err := d2latex.Measure(obj.Text().Text)
+				if err != nil {
+					return err
+				}
+				dims = d2target.NewTextDimensions(width, height)
+			} else {
+				var err error
+				dims, err = getMarkdownDimensions(mtexts, ruler, obj.Text())
+				if err != nil {
+					return err
+				}
 			}
 			innerLabelPadding = 0
 		} else {
@@ -1048,6 +1110,7 @@ var ReservedKeywords = map[string]struct{}{
 	"near":       {},
 	"width":      {},
 	"height":     {},
+	"direction":  {},
 }
 
 // ReservedKeywordHolders are reserved keywords that are meaningless on its own and exist solely to hold a set of reserved keywords
