@@ -7,9 +7,11 @@ import (
 	"strings"
 	"unicode"
 
-	"oss.terrastruct.com/xdefer"
+	"oss.terrastruct.com/util-go/xdefer"
 
-	"oss.terrastruct.com/xrand"
+	"oss.terrastruct.com/util-go/xrand"
+
+	"oss.terrastruct.com/util-go/go2"
 
 	"oss.terrastruct.com/d2/d2ast"
 	"oss.terrastruct.com/d2/d2compiler"
@@ -17,7 +19,6 @@ import (
 	"oss.terrastruct.com/d2/d2graph"
 	"oss.terrastruct.com/d2/d2parser"
 	"oss.terrastruct.com/d2/d2target"
-	"oss.terrastruct.com/d2/lib/go2"
 )
 
 func Create(g *d2graph.Graph, key string) (_ *d2graph.Graph, newKey string, err error) {
