@@ -10,6 +10,7 @@ import (
 	"fmt"
 	"hash/fnv"
 	"io"
+	"sort"
 	"strings"
 
 	"math"
@@ -19,14 +20,17 @@ import (
 	"github.com/alecthomas/chroma/lexers"
 	"github.com/alecthomas/chroma/styles"
 
+	"oss.terrastruct.com/util-go/go2"
+
+	"oss.terrastruct.com/d2/d2graph"
 	"oss.terrastruct.com/d2/d2renderers/d2fonts"
-	"oss.terrastruct.com/d2/d2renderers/textmeasure"
+	"oss.terrastruct.com/d2/d2renderers/d2latex"
 	"oss.terrastruct.com/d2/d2target"
 	"oss.terrastruct.com/d2/lib/color"
 	"oss.terrastruct.com/d2/lib/geo"
-	"oss.terrastruct.com/d2/lib/go2"
 	"oss.terrastruct.com/d2/lib/label"
 	"oss.terrastruct.com/d2/lib/shape"
+	"oss.terrastruct.com/d2/lib/textmeasure"
 )
 
 const (
@@ -334,26 +338,16 @@ func pathData(connection d2target.Connection, idToShape map[string]d2target.Shap
 	return strings.Join(path, " ")
 }
 
-func labelMask(id string, connection d2target.Connection, labelTL, tl, br *geo.Point) string {
-	width := br.X - tl.X
-	height := br.Y - tl.Y
-	return strings.Join([]string{
-		fmt.Sprintf(`<mask id="%s" maskUnits="userSpaceOnUse" x="%f" y="%f" width="%f" height="%f">`,
-			id, tl.X, tl.Y, width, height,
-		),
-		fmt.Sprintf(`<rect x="%f" y="%f" width="%f" height="%f" fill="white"></rect>`,
-			tl.X, tl.Y, width, height,
-		),
-		fmt.Sprintf(`<rect x="%f" y="%f" width="%d" height="%d" fill="black"></rect>`,
-			labelTL.X, labelTL.Y,
-			connection.LabelWidth,
-			connection.LabelHeight,
-		),
-		`</mask>`,
-	}, "\n")
+func makeLabelMask(labelTL *geo.Point, width, height int) string {
+	return fmt.Sprintf(`<rect x="%f" y="%f" width="%d" height="%d" fill="black"></rect>`,
+		labelTL.X, labelTL.Y,
+		width,
+		height,
+	)
 }
 
-func drawConnection(writer io.Writer, connection d2target.Connection, markers map[string]struct{}, idToShape map[string]d2target.Shape) {
+func drawConnection(writer io.Writer, labelMaskID string, connection d2target.Connection, markers map[string]struct{}, idToShape map[string]d2target.Shape) (labelMask string) {
+	fmt.Fprintf(writer, `<g id="%s">`, escapeText(connection.ID))
 	var markerStart string
 	if connection.SrcArrow != d2target.NoArrowhead {
 		id := arrowheadMarkerID(false, connection)
@@ -383,7 +377,6 @@ func drawConnection(writer io.Writer, connection d2target.Connection, markers ma
 	}
 
 	var labelTL *geo.Point
-	var mask string
 	if connection.Label != "" {
 		labelTL = connection.GetLabelTopLeft()
 		labelTL.X = math.Round(labelTL.X)
@@ -416,18 +409,16 @@ func drawConnection(writer io.Writer, connection d2target.Connection, markers ma
 			br.X = math.Max(br.X, labelTL.X+float64(connection.LabelWidth))
 			br.Y = math.Max(br.Y, labelTL.Y+float64(connection.LabelHeight))
 
-			maskID := fmt.Sprintf("mask-%s", hash(connection.ID))
-			fmt.Fprint(writer, labelMask(maskID, connection, labelTL, tl, br))
-			mask = fmt.Sprintf(`mask="url(#%s)" `, maskID)
+			labelMask = makeLabelMask(labelTL, connection.LabelWidth, connection.LabelHeight)
 		}
 	}
 
-	fmt.Fprintf(writer, `<path d="%s" class="connection" style="fill:none;%s" %s%s%s/>`,
+	fmt.Fprintf(writer, `<path d="%s" class="connection" style="fill:none;%s" %s%smask="url(#%s)"/>`,
 		pathData(connection, idToShape),
 		connectionStyle(connection),
 		markerStart,
 		markerEnd,
-		mask,
+		labelMaskID,
 	)
 
 	if connection.Label != "" {
@@ -437,7 +428,11 @@ func drawConnection(writer io.Writer, connection d2target.Connection, markers ma
 		} else if connection.Italic {
 			fontClass += "-italic"
 		}
-		textStyle := fmt.Sprintf("text-anchor:%s;font-size:%vpx;fill:%s", "middle", connection.FontSize, "black")
+		fontColor := "black"
+		if connection.Color != "" {
+			fontColor = connection.Color
+		}
+		textStyle := fmt.Sprintf("text-anchor:%s;font-size:%vpx;fill:%s", "middle", connection.FontSize, fontColor)
 		x := labelTL.X + float64(connection.LabelWidth)/2
 		y := labelTL.Y + float64(connection.FontSize)
 		fmt.Fprintf(writer, `<text class="%s" x="%f" y="%f" style="%s">%s</text>`,
@@ -447,6 +442,41 @@ func drawConnection(writer io.Writer, connection d2target.Connection, markers ma
 			renderText(connection.Label, x, float64(connection.LabelHeight)),
 		)
 	}
+
+	length := geo.Route(connection.Route).Length()
+	if connection.SrcLabel != "" {
+		// TODO use arrowhead label dimensions https://github.com/terrastruct/d2/issues/183
+		size := float64(connection.FontSize)
+		position := 0.
+		if length > 0 {
+			position = size / length
+		}
+		fmt.Fprint(writer, renderArrowheadLabel(connection, connection.SrcLabel, position, size, size))
+	}
+	if connection.DstLabel != "" {
+		// TODO use arrowhead label dimensions https://github.com/terrastruct/d2/issues/183
+		size := float64(connection.FontSize)
+		position := 1.
+		if length > 0 {
+			position -= size / length
+		}
+		fmt.Fprint(writer, renderArrowheadLabel(connection, connection.DstLabel, position, size, size))
+	}
+	fmt.Fprintf(writer, `</g>`)
+	return
+}
+
+func renderArrowheadLabel(connection d2target.Connection, text string, position, width, height float64) string {
+	labelTL := label.UnlockedTop.GetPointOnRoute(connection.Route, float64(connection.StrokeWidth), position, width, height)
+
+	textStyle := fmt.Sprintf("text-anchor:%s;font-size:%vpx;fill:%s", "middle", connection.FontSize, "black")
+	x := labelTL.X + width/2
+	y := labelTL.Y + float64(connection.FontSize)
+	return fmt.Sprintf(`<text class="text-italic" x="%f" y="%f" style="%s">%s</text>`,
+		x, y,
+		textStyle,
+		renderText(text, x, height),
+	)
 }
 
 func renderOval(tl *geo.Point, width, height float64, style string) string {
@@ -469,7 +499,93 @@ func defineShadowFilter(writer io.Writer) {
 </defs>`)
 }
 
-func drawShape(writer io.Writer, targetShape d2target.Shape) error {
+func render3dRect(targetShape d2target.Shape) string {
+	moveTo := func(p d2target.Point) string {
+		return fmt.Sprintf("M%d,%d", p.X+targetShape.Pos.X, p.Y+targetShape.Pos.Y)
+	}
+	lineTo := func(p d2target.Point) string {
+		return fmt.Sprintf("L%d,%d", p.X+targetShape.Pos.X, p.Y+targetShape.Pos.Y)
+	}
+
+	// draw border all in one path to prevent overlapping sections
+	var borderSegments []string
+	borderSegments = append(borderSegments,
+		moveTo(d2target.Point{X: 0, Y: 0}),
+	)
+	for _, v := range []d2target.Point{
+		{X: threeDeeOffset, Y: -threeDeeOffset},
+		{X: targetShape.Width + threeDeeOffset, Y: -threeDeeOffset},
+		{X: targetShape.Width + threeDeeOffset, Y: targetShape.Height - threeDeeOffset},
+		{X: targetShape.Width, Y: targetShape.Height},
+		{X: 0, Y: targetShape.Height},
+		{X: 0, Y: 0},
+		{X: targetShape.Width, Y: 0},
+		{X: targetShape.Width, Y: targetShape.Height},
+	} {
+		borderSegments = append(borderSegments, lineTo(v))
+	}
+	// move to top right to draw last segment without overlapping
+	borderSegments = append(borderSegments,
+		moveTo(d2target.Point{X: targetShape.Width, Y: 0}),
+	)
+	borderSegments = append(borderSegments,
+		lineTo(d2target.Point{X: targetShape.Width + threeDeeOffset, Y: -threeDeeOffset}),
+	)
+	border := targetShape
+	border.Fill = "none"
+	borderStyle := shapeStyle(border)
+	renderedBorder := fmt.Sprintf(`<path d="%s" style="%s"/>`,
+		strings.Join(borderSegments, " "), borderStyle)
+
+	// create mask from border stroke, to cut away from the shape fills
+	maskID := fmt.Sprintf("border-mask-%v", escapeText(targetShape.ID))
+	borderMask := strings.Join([]string{
+		fmt.Sprintf(`<defs><mask id="%s" maskUnits="userSpaceOnUse" x="%d" y="%d" width="%d" height="%d">`,
+			maskID, targetShape.Pos.X, targetShape.Pos.Y-threeDeeOffset, targetShape.Width+threeDeeOffset, targetShape.Height+threeDeeOffset,
+		),
+		fmt.Sprintf(`<rect x="%d" y="%d" width="%d" height="%d" fill="white"></rect>`,
+			targetShape.Pos.X, targetShape.Pos.Y-threeDeeOffset, targetShape.Width+threeDeeOffset, targetShape.Height+threeDeeOffset,
+		),
+		fmt.Sprintf(`<path d="%s" style="%s;stroke:#000;fill:none;opacity:1;"/></mask></defs>`,
+			strings.Join(borderSegments, ""), borderStyle),
+	}, "\n")
+
+	// render the main rectangle without stroke and the border mask
+	mainShape := targetShape
+	mainShape.Stroke = "none"
+	mainRect := fmt.Sprintf(`<rect x="%d" y="%d" width="%d" height="%d" style="%s" mask="url(#%s)"/>`,
+		targetShape.Pos.X, targetShape.Pos.Y, targetShape.Width, targetShape.Height, shapeStyle(mainShape), maskID,
+	)
+
+	// render the side shapes in the darkened color without stroke and the border mask
+	var sidePoints []string
+	for _, v := range []d2target.Point{
+		{X: 0, Y: 0},
+		{X: threeDeeOffset, Y: -threeDeeOffset},
+		{X: targetShape.Width + threeDeeOffset, Y: -threeDeeOffset},
+		{X: targetShape.Width + threeDeeOffset, Y: targetShape.Height - threeDeeOffset},
+		{X: targetShape.Width, Y: targetShape.Height},
+		{X: targetShape.Width, Y: 0},
+	} {
+		sidePoints = append(sidePoints,
+			fmt.Sprintf("%d,%d", v.X+targetShape.Pos.X, v.Y+targetShape.Pos.Y),
+		)
+	}
+	darkerColor, err := color.Darken(targetShape.Fill)
+	if err != nil {
+		darkerColor = targetShape.Fill
+	}
+	sideShape := targetShape
+	sideShape.Fill = darkerColor
+	sideShape.Stroke = "none"
+	renderedSides := fmt.Sprintf(`<polygon points="%s" style="%s" mask="url(#%s)"/>`,
+		strings.Join(sidePoints, " "), shapeStyle(sideShape), maskID)
+
+	return borderMask + mainRect + renderedSides + renderedBorder
+}
+
+func drawShape(writer io.Writer, targetShape d2target.Shape) (labelMask string, err error) {
+	fmt.Fprintf(writer, `<g id="%s">`, escapeText(targetShape.ID))
 	tl := geo.NewPoint(float64(targetShape.Pos.X), float64(targetShape.Pos.Y))
 	width := float64(targetShape.Width)
 	height := float64(targetShape.Height)
@@ -490,7 +606,12 @@ func drawShape(writer io.Writer, targetShape d2target.Shape) error {
 		}
 	}
 
-	fmt.Fprintf(writer, `<g class="shape" %s>`, shadowAttr)
+	var blendModeClass string
+	if targetShape.Blend {
+		blendModeClass = " blend"
+	}
+
+	fmt.Fprintf(writer, `<g class="shape%s" %s>`, blendModeClass, shadowAttr)
 
 	var multipleTL *geo.Point
 	if targetShape.Multiple {
@@ -500,12 +621,12 @@ func drawShape(writer io.Writer, targetShape d2target.Shape) error {
 	switch targetShape.Type {
 	case d2target.ShapeClass:
 		drawClass(writer, targetShape)
-		fmt.Fprintf(writer, `</g>`)
-		return nil
+		fmt.Fprintf(writer, `</g></g>`)
+		return labelMask, nil
 	case d2target.ShapeSQLTable:
 		drawTable(writer, targetShape)
-		fmt.Fprintf(writer, `</g>`)
-		return nil
+		fmt.Fprintf(writer, `</g></g>`)
+		return labelMask, nil
 	case d2target.ShapeOval:
 		if targetShape.Multiple {
 			fmt.Fprint(writer, renderOval(multipleTL, width, height, style))
@@ -518,49 +639,17 @@ func drawShape(writer io.Writer, targetShape d2target.Shape) error {
 			targetShape.Pos.X, targetShape.Pos.Y, targetShape.Width, targetShape.Height, style)
 
 	// TODO should standardize "" to rectangle
-	case d2target.ShapeRectangle, "":
+	case d2target.ShapeRectangle, d2target.ShapeSequenceDiagram, "":
 		if targetShape.ThreeDee {
-			darkerColor, err := color.Darken(targetShape.Fill)
-			if err != nil {
-				darkerColor = targetShape.Fill
+			fmt.Fprint(writer, render3dRect(targetShape))
+		} else {
+			if targetShape.Multiple {
+				fmt.Fprintf(writer, `<rect x="%d" y="%d" width="%d" height="%d" style="%s" />`,
+					targetShape.Pos.X+10, targetShape.Pos.Y-10, targetShape.Width, targetShape.Height, style)
 			}
-
-			var topPolygonPoints []string
-			for _, v := range []d2target.Point{
-				{X: 0, Y: 0},
-				{X: threeDeeOffset, Y: -1 * threeDeeOffset},
-				{X: targetShape.Width + threeDeeOffset, Y: -1 * threeDeeOffset},
-				{X: targetShape.Width, Y: 0},
-				{X: 0, Y: 0},
-			} {
-				topPolygonPoints = append(topPolygonPoints,
-					fmt.Sprintf("%d,%d ", v.X+targetShape.Pos.X, v.Y+targetShape.Pos.Y),
-				)
-			}
-			fmt.Fprintf(writer, `<polygon points="%s" style="fill:%s;"/>`,
-				strings.Join(topPolygonPoints, ""), darkerColor)
-
-			var rightPolygonPoints []string
-			for _, v := range []d2target.Point{
-				{X: targetShape.Width, Y: 0},
-				{X: targetShape.Width + threeDeeOffset, Y: -1 * threeDeeOffset},
-				{X: targetShape.Width + threeDeeOffset, Y: targetShape.Height - threeDeeOffset},
-				{X: targetShape.Width, Y: targetShape.Height},
-			} {
-				rightPolygonPoints = append(rightPolygonPoints,
-					fmt.Sprintf("%d,%d ", v.X+targetShape.Pos.X, v.Y+targetShape.Pos.Y),
-				)
-			}
-			fmt.Fprintf(writer, `<polygon points="%s" style="fill:%s;"/>`,
-				strings.Join(rightPolygonPoints, ""), darkerColor)
-		}
-		if targetShape.Multiple {
 			fmt.Fprintf(writer, `<rect x="%d" y="%d" width="%d" height="%d" style="%s" />`,
-				targetShape.Pos.X+10, targetShape.Pos.Y-10, targetShape.Width, targetShape.Height, style)
+				targetShape.Pos.X, targetShape.Pos.Y, targetShape.Width, targetShape.Height, style)
 		}
-		fmt.Fprintf(writer, `<rect x="%d" y="%d" width="%d" height="%d" style="%s" />`,
-			targetShape.Pos.X, targetShape.Pos.Y, targetShape.Width, targetShape.Height, style)
-
 	case d2target.ShapeText, d2target.ShapeCode:
 	default:
 		if targetShape.Multiple {
@@ -619,19 +708,19 @@ func drawShape(writer io.Writer, targetShape d2target.Shape) error {
 		case d2target.ShapeCode:
 			lexer := lexers.Get(targetShape.Language)
 			if lexer == nil {
-				return fmt.Errorf("code snippet lexer for %s not found", targetShape.Language)
+				return labelMask, fmt.Errorf("code snippet lexer for %s not found", targetShape.Language)
 			}
 			style := styles.Get("github")
 			if style == nil {
-				return errors.New(`code snippet style "github" not found`)
+				return labelMask, errors.New(`code snippet style "github" not found`)
 			}
 			formatter := formatters.Get("svg")
 			if formatter == nil {
-				return errors.New(`code snippet formatter "svg" not found`)
+				return labelMask, errors.New(`code snippet formatter "svg" not found`)
 			}
 			iterator, err := lexer.Tokenise(nil, targetShape.Label)
 			if err != nil {
-				return err
+				return labelMask, err
 			}
 
 			svgStyles := styleToSVG(style)
@@ -658,20 +747,34 @@ func drawShape(writer io.Writer, targetShape d2target.Shape) error {
 			}
 			fmt.Fprintf(writer, "</g></g>")
 		case d2target.ShapeText:
-			render, err := textmeasure.RenderMarkdown(targetShape.Label)
-			if err != nil {
-				return err
+			if targetShape.Language == "latex" {
+				render, err := d2latex.Render(targetShape.Label)
+				if err != nil {
+					return labelMask, err
+				}
+				fmt.Fprintf(writer, `<g transform="translate(%f %f)" style="opacity:%f">`, box.TopLeft.X, box.TopLeft.Y, targetShape.Opacity)
+				fmt.Fprint(writer, render)
+				fmt.Fprintf(writer, "</g>")
+			} else {
+				render, err := textmeasure.RenderMarkdown(targetShape.Label)
+				if err != nil {
+					return labelMask, err
+				}
+				fmt.Fprintf(writer, `<g><foreignObject requiredFeatures="http://www.w3.org/TR/SVG11/feature#Extensibility" x="%f" y="%f" width="%d" height="%d">`,
+					box.TopLeft.X, box.TopLeft.Y, targetShape.Width, targetShape.Height,
+				)
+				// we need the self closing form in this svg/xhtml context
+				render = strings.ReplaceAll(render, "<hr>", "<hr />")
+				fmt.Fprintf(writer, `<div xmlns="http://www.w3.org/1999/xhtml" class="md">%v</div>`, render)
+				fmt.Fprint(writer, `</foreignObject></g>`)
 			}
-			fmt.Fprintf(writer, `<g><foreignObject requiredFeatures="http://www.w3.org/TR/SVG11/feature#Extensibility" x="%f" y="%f" width="%d" height="%d">`,
-				box.TopLeft.X, box.TopLeft.Y, targetShape.Width, targetShape.Height,
-			)
-			// we need the self closing form in this svg/xhtml context
-			render = strings.ReplaceAll(render, "<hr>", "<hr />")
-			fmt.Fprintf(writer, `<div xmlns="http://www.w3.org/1999/xhtml" class="md">%v</div>`, render)
-			fmt.Fprint(writer, `</foreignObject></g>`)
 		default:
-			textStyle := fmt.Sprintf("text-anchor:%s;font-size:%vpx;fill:%s", "middle", targetShape.FontSize, "black")
-			x := labelTL.X + float64(targetShape.LabelWidth)/2
+			fontColor := "black"
+			if targetShape.Color != "" {
+				fontColor = targetShape.Color
+			}
+			textStyle := fmt.Sprintf("text-anchor:%s;font-size:%vpx;fill:%s", "middle", targetShape.FontSize, fontColor)
+			x := labelTL.X + float64(targetShape.LabelWidth)/2.
 			// text is vertically positioned at its baseline which is at labelTL+FontSize
 			y := labelTL.Y + float64(targetShape.FontSize)
 			fmt.Fprintf(writer, `<text class="%s" x="%f" y="%f" style="%s">%s</text>`,
@@ -680,9 +783,13 @@ func drawShape(writer io.Writer, targetShape d2target.Shape) error {
 				textStyle,
 				renderText(targetShape.Label, x, float64(targetShape.LabelHeight)),
 			)
+			if targetShape.Blend {
+				labelMask = makeLabelMask(labelTL, targetShape.LabelWidth, targetShape.LabelHeight-d2graph.INNER_LABEL_PADDING)
+			}
 		}
 	}
-	return nil
+	fmt.Fprintf(writer, `</g>`)
+	return labelMask, nil
 }
 
 func escapeText(text string) string {
@@ -719,6 +826,10 @@ func shapeStyle(shape d2target.Shape) string {
 	out += fmt.Sprintf(`stroke:%s;`, shape.Stroke)
 	out += fmt.Sprintf(`opacity:%f;`, shape.Opacity)
 	out += fmt.Sprintf(`stroke-width:%d;`, shape.StrokeWidth)
+	if shape.StrokeDash != 0 {
+		dashSize, gapSize := getStrokeDashAttributes(float64(shape.StrokeWidth), shape.StrokeDash)
+		out += fmt.Sprintf(`stroke-dasharray:%f,%f;`, dashSize, gapSize)
+	}
 
 	return out
 }
@@ -840,7 +951,7 @@ func embedFonts(buf *bytes.Buffer) {
 // TODO minify output at end
 func Render(diagram *d2target.Diagram) ([]byte, error) {
 	buf := &bytes.Buffer{}
-	_, _ = setViewbox(buf, diagram)
+	w, h := setViewbox(buf, diagram)
 
 	buf.WriteString(fmt.Sprintf(`<style type="text/css">
 <![CDATA[
@@ -867,28 +978,59 @@ func Render(diagram *d2target.Diagram) ([]byte, error) {
 		}
 	}
 
+	// Mask URLs are global. So when multiple SVGs attach to a DOM, they share
+	// the same namespace for mask URLs.
+	labelMaskID, err := diagram.HashID()
+	if err != nil {
+		return nil, err
+	}
+
 	// SVG has no notion of z-index. The z-index is effectively the order it's drawn.
 	// So draw from the least nested to most nested
 	idToShape := make(map[string]d2target.Shape)
-	highest := 1
+	allObjects := make([]DiagramObject, 0, len(diagram.Shapes)+len(diagram.Connections))
 	for _, s := range diagram.Shapes {
-		highest = go2.Max(highest, s.Level)
 		idToShape[s.ID] = s
+		allObjects = append(allObjects, s)
 	}
-	for i := 1; i <= highest; i++ {
-		for _, s := range diagram.Shapes {
-			if s.Level == i {
-				err := drawShape(buf, s)
-				if err != nil {
-					return nil, err
-				}
+	for _, c := range diagram.Connections {
+		allObjects = append(allObjects, c)
+	}
+
+	sortObjects(allObjects)
+
+	var labelMasks []string
+	markers := map[string]struct{}{}
+	for _, obj := range allObjects {
+		if c, is := obj.(d2target.Connection); is {
+			labelMask := drawConnection(buf, labelMaskID, c, markers, idToShape)
+			if labelMask != "" {
+				labelMasks = append(labelMasks, labelMask)
 			}
+		} else if s, is := obj.(d2target.Shape); is {
+			labelMask, err := drawShape(buf, s)
+			if err != nil {
+				return nil, err
+			} else if labelMask != "" {
+				labelMasks = append(labelMasks, labelMask)
+			}
+		} else {
+			return nil, fmt.Errorf("unknow object of type %T", obj)
 		}
 	}
 
-	markers := map[string]struct{}{}
-	for _, c := range diagram.Connections {
-		drawConnection(buf, c, markers, idToShape)
+	if len(labelMasks) > 0 {
+		fmt.Fprint(buf, strings.Join([]string{
+			fmt.Sprintf(`<mask id="%s" maskUnits="userSpaceOnUse" x="0" y="0" width="%d" height="%d">`,
+				labelMaskID, w, h,
+			),
+			fmt.Sprintf(`<rect x="0" y="0" width="%d" height="%d" fill="white"></rect>`,
+				w,
+				h,
+			),
+			strings.Join(labelMasks, "\n"),
+			`</mask>`,
+		}, "\n"))
 	}
 
 	embedFonts(buf)
@@ -897,6 +1039,39 @@ func Render(diagram *d2target.Diagram) ([]byte, error) {
 	return buf.Bytes(), nil
 }
 
+type DiagramObject interface {
+	GetID() string
+	GetZIndex() int
+}
+
+// sortObjects sorts all diagrams objects (shapes and connections) in the desired drawing order
+// the sorting criteria is:
+// 1. zIndex, lower comes first
+// 2. two shapes with the same zIndex are sorted by their level (container nesting), containers come first
+// 3. two shapes with the same zIndex and same level, are sorted in the order they were exported
+// 4. shape and edge, shapes come first
+func sortObjects(allObjects []DiagramObject) {
+	sort.SliceStable(allObjects, func(i, j int) bool {
+		// first sort by zIndex
+		iZIndex := allObjects[i].GetZIndex()
+		jZIndex := allObjects[j].GetZIndex()
+		if iZIndex != jZIndex {
+			return iZIndex < jZIndex
+		}
+
+		// then, if both are shapes, parents come before their children
+		iShape, iIsShape := allObjects[i].(d2target.Shape)
+		jShape, jIsShape := allObjects[j].(d2target.Shape)
+		if iIsShape && jIsShape {
+			return iShape.Level < jShape.Level
+		}
+
+		// then, shapes come before connections
+		_, jIsConnection := allObjects[j].(d2target.Connection)
+		return iIsShape && jIsConnection
+	})
+}
+
 func hash(s string) string {
 	const secret = "lalalas"
 	h := fnv.New32a()
